@@ -11,7 +11,7 @@ __PACKAGE__->table('event');
 __PACKAGE__->add_columns(
   id => { data_type => 'integer', is_auto_increment => 1 },
   starts_at => { data_type => 'datetime', extra => { timezone => "America/Chicago" } },
-  created_on => { data_type => 'timestamp', extra => { timezone => "America/Chicago" } },
+  created_on => { data_type => 'timestamp', extra => { timezone => "America/Chicago", floating_tz_ok => 1 } },
 );
 
 __PACKAGE__->set_primary_key('id');
